/*
 * Vasilca Marian Laurentiu
 * 332CB
 */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/sleep.h>
#include <avr/wdt.h>
#include <string.h>
#include <stdio.h>
#include "lcd.h"


#define F_CPU 16000000

#define PAUSE_MAX_DISTANCE  30


// C  D  E  F  G   A  B  C
// DO RE MI FA SOL LA SI DO
#define C 119
#define D 106
#define E 94
#define F 89
#define G 79
#define A 70
#define B 63
#define Cm 59
#define melody_one_size 57
#define melody_two_size 50
    

static uint8_t melody_one[melody_one_size] = {C, C, C, G, A, A, A, G,
                    E, E, D, D, C, G, C, C, C, G, A, A, G,
                    E, E, D, D, C, G, G, C, C, C, G, G, C, C, C,
                    C, C, C, C, C, C, C, C, C, C, C, C, C, C,
                    C, C, C, G, A, A, G};

static uint8_t melody_two[melody_two_size] = {A, A, B, A, D, C,
                                            A, A, B, A, E, D,
                                            A, A, A, F, D, C, B,
                                            G, G, F, D, E, D,
                                            A, A, B, A, D, C,
                                            A, A, B, A, E, D, 
                                            A, A, A, F, D, C, B,
                                            G, G, F, D, E, D};                    

void LCD_clear_top_line(){
    LCD_printAt(0x00,"                "); 
}
void LCD_clear_bottom_line(){
    LCD_printAt(0x40,"                "); 
}

/* 
 *Four-Character Code
 * Folosit pentru a indentifica formate de date
 */
#define FCC(c1, c2, c3, c4) \
    (((DWORD)c4 << 24) + ((DWORD)c3 << 16) + ((WORD)c2 << 8) + (BYTE)c1)

unsigned char Buff1[256];   // Wave output FIFO
unsigned char Buff2[256];   // Wave output FIFO 
volatile uint8_t read = 1;
volatile uint16_t counter = 0;
volatile uint8_t sync = 0;

int music_files_no = 0;
int current_music_file = -1;

uint16_t miliseconds = 0;
uint8_t minutes = 0, seconds = 0, hours = 0;
char time_string[20];

char states[5][10] = { "STOP", "PLAY", "PAUSE","FIRST", "SECOND" };

volatile unsigned long int distance = 0;
volatile int dd = 0;
volatile unsigned long int microseconds = 0;

#define NONE -1
#define BTN_PB0 1
#define BTN_PB1 2
#define STOP    0
#define PLAY    1
#define PAUSE   2
#define NEXT    3
#define SECOND 4
#define FIRST 3

//controleaza redarea audio
volatile uint8_t continue_play = PLAY;
//memoreaza ce buton a fost apasat
volatile uint8_t button_pressed = NONE;

int DIST_get();


ISR(TIMER2_COMPA_vect)
{
    microseconds ++;
}

void Timer0_init(void)
{
        /* Seteaza difuzorul ca output */
    DDRD |= (1 << PD5);
    TCCR1A |= (1 << WGM10) | (1 << COM1A0);
    TCCR1B |= (1 << WGM13) | (1 << CS12);

    /* Seteaza modul CTC */
    TCCR2A = (1 << WGM21);

    /* Seteaza prescaller-ul. */
    TCCR2B |= (1 << CS22) | (1 << CS21) | (1 << CS20);

    /* Seteaza OCR; la fiecare 10 ms. */
    OCR2A = 156;

    /* Seteaza intreruperile de timer. */
    TIMSK1 |= (1 << OCIE1A);
}

void Timer1_init(void)
{
    OCR1A = 0xFF;
    TCCR1A = (1 << COM1A1) | (1 << COM1A0) | (1 << WGM10);  // mod fastPWM 8-bit; set OC0A on match, clear on bottom
    TCCR1B = (1 << WGM12) | (1 << CS10);            // prescaler 8
}

void Timer2_init(void)
{
    OCR2A = 0x10;
    TCCR2A = (1 << WGM21);          // mod CTC
    TCCR2B = (1 << CS21) | (1 << CS20);         // prescaler 1
    TIMSK2 = (1 << OCIE2A);         // intrerupere de compare pe A 
}

void IO_init(void)
{
    /* setez pinul PA0 ca intrare */
    DDRB &= ~(1 << PB0);
    /* activez rezistenta de pull-up pentru pinul PB0*/
    PORTB |= (1 << PB0);

        /* setez pinul PA0 ca intrare */
    DDRB &= ~(1 << PB1);
    /* activez rezistenta de pull-up pentru pinul PB0*/
    PORTB |= (1 << PB1);

        /* setez pinul PA0 ca intrare */
    DDRB &= ~(1 << PB2);
    /* activez rezistenta de pull-up pentru pinul PB0*/
    PORTB |= (1 << PB2);

    /* setez pinul PC0 ca iesire */
    DDRD |= (1 << PD0);

    /* setez pinul PC0 ca iesire */
    DDRD |= (1 << PD1);

    /* setez pinul PC0 ca iesire */
    DDRD |= (1 << PD2);


    // DIFUZOR
    /* Seteaza difuzorul ca output */
    DDRD |= (1 << PD5);

}


void printState(int state) {
    LCD_clear_bottom_line();
    LCD_printAt(0x40,states[state]);
}

ISR(INT2_vect) {
    if((PINB & (1 << PB2)) == 0 ) {
        dd = 2;
        distance = microseconds;
    } else {
        microseconds = 0;
        dd = 1;
    }
}

void DIST_init() {
    EIMSK |= (1 << INT2);
    EICRA |= (1 << ISC20);

    DDRD |= (1 << PD6);
    DDRB &= ~(1 << PB2);
    PORTB |= (1 << PB2);

}

int DIST_get() {

    dd = 0;
    distance = 0;
    PORTD &= ~(1 << PD6);
    _delay_us(2);
    PORTD |= (1 << PD6);
    _delay_us(10);
    PORTD &= ~(1 << PD6);

    while(dd != 2) {}

    return distance;
}

#if 0
int main (void)
{
    int i=0, j=0;

    DIST_init();
    sei();
    IO_init();
    LCD_init();
    Timer2_init();

    LCD_clear_top_line();
    
    while(1){

         LCD_clear_top_line();
        LCD_printAt(0x00,"MUSIC PLAYER");
         _delay_ms(500);

        while(DIST_get() < PAUSE_MAX_DISTANCE) {
            printState(PAUSE);
            _delay_ms(2000);
        }
        LCD_clear_bottom_line();

    }//main loop
    
    return 0;
}
#endif

#if 1
int main(void) {

    int i = 0;

    LCD_init();
    IO_init();
    LCD_clear_top_line();
    
    Timer0_init();


while(1) {

        for (i = 0; i < melody_two_size; i++) {

            if (melody_two[i] > 50)
                PORTD |= (1 << PD0);
            if (melody_two[i] > 70)
                PORTD |= (1 << PD1);
            if (melody_two[i] > 100)
                PORTD |= (1 << PD2);

            OCR1A = melody_two[i];
            TCCR1A |= (1 << COM1A0);
            _delay_ms(500);

            TCCR1A &= ~(1 << COM1A0);
            PORTD &= ~(1 << PD2);
            PORTD &= ~(1 << PD1);
            PORTD &= ~(1 << PD0);
        }
    
    }
    
    return 0;
}
#endif